/*
 * Caelum dome
 */
#ifndef __CAELUMDOME_H__
#define __CAELUMDOME_H__

#include "Ogre.h"

// Predeclaration required
class CaelumManager;

using namespace Ogre;

#define DOME_TUS_STARS              0
#define DOME_TUS_COLOUR             1
#define DOME_TUS_SUN                2

class CaelumDome {
	public:
    CaelumDome (CaelumManager *parent);
    // Set the dome colour map
    bool setColourMap (String file);
    // Set the dome colour
    void setColour (ColourValue &colour);
    void setColour (Real r, Real g, Real b);
    // Get the dome colour
    ColourValue &getColour (void);
    // Set the pressure
    void setPressure (Real pressure);
    // Get the pressure
    Real getPressure (void);
    // Update
    void _update (void);
	protected:
		// Parent
		CaelumManager *mParent;
    // Image resource for automatic colours
    Image *mColourMap;
    // Data extracted from the colour map
    uchar *mColourMapData;
    // Number of steps stored in the colour map (number of pixels)
    size_t mColourMapDataSteps;
    // Number of colour components
    unsigned short int mColourMapDataComponents;
    // Sky colour
    ColourValue mColour;
    // Air pressure
    Real mPressure;
    // Node associated with the dome
    SceneNode *mNode;
    // Dynamic sky dome material
    Material *mMaterial;
    // Touched flag
    bool mTouched;
};

#endif