/*
 * Caelum haze
 * Class encapsulating the data and behavior of haze
 * Author: Kencho
 * Date: 03-09-2004
 */
#ifndef __CAELUMHAZE_H__
#define __CAELUMHAZE_H__

#include "Ogre.h"

// Predeclaration required
class CaelumManager;

using namespace Ogre;

class CaelumHaze {
  public:
    // Constructor
    CaelumHaze (CaelumManager *parent) {
      mParent = parent;

      // Init with default values
      mColourMap = 0;
      mColourMapData = 0;
      mColourMapDataSteps = 0;
      mColourMapDataComponents = 0;
      mColour = ColourValue::White;
      mTurbidity = 0.01;
      mMaximumTurbidity = 0.001;
      mMinimumTurbidity = 0.0005;

      mTouched = true;
    }
    // Destructor
    ~CaelumHaze () {
      // Unload resources
      if (mColourMap) {
        delete mColourMap;
      }
    }
    // Set the haze colour map
    bool setColourMap (String file);
    // Set the haze colour
    void setColour (ColourValue &colour);
    void setColour (Real r, Real g, Real b);
    // Get the haze colour
    ColourValue &getColour (void);
    // Set the haze turbidity
    void setTurbidity (Real turbidity);
    // Get the haze turbidity
    Real getTurbidity (void);
    // Get the fog range
    Real getRange (void);
    // Update the scene haze
    void _update (void);
  protected:
    // Parent - mediator
    CaelumManager *mParent;
    // Image resource for automatic colours
    Image *mColourMap;
    // Data extracted from the colour map
    uchar *mColourMapData;
    // Number of steps stored in the colour map (number of pixels)
    size_t mColourMapDataSteps;
    // Number of colour components
    unsigned short int mColourMapDataComponents;
    // Haze colour
    ColourValue mColour;
    // Haze turbidity
    Real mTurbidity;
    // Haze turbidity limits
    Real mMaximumTurbidity;
    Real mMinimumTurbidity;
    // Touched flag
    bool mTouched;
    // Calculate the new colour
    void _recalculateColour (void);
};

#endif