/*
 * CaelumManager.h
 */
#ifndef __CAELUMMANAGER_H__
#define __CAELUMMANAGER_H__

#include "Ogre.h"

#include "CaelumHaze.h"
#include "CaelumDome.h"
#include "CaelumSun.h"

using namespace Ogre;

class CaelumManager {
  friend class CaelumHaze;
  friend class CaelumDome;
  friend class CaelumSun;
  public:
    // Constructor
    CaelumManager (SceneManager *scnMgr) {
      // Scene Manager
      mSceneMgr = scnMgr;
      // Camera
      mCamera = 0;
      // Haze
      mHaze = new CaelumHaze (this);
      // Dome
      mDome = new CaelumDome (this);
      // Sun
      mSun = new CaelumSun (this);
      // Set base distance
      mMetreUnit = 1;
      // Equator parallel & Greenwich meridian
      mLocation = Vector3 (0, 0, 0);
      // 00hrs 00' 00"
      mTime = 0.0;
      // Update all
      mUpdateFlags = !0;
      // Difference this from any other object in the scene -- Mark Caelum as a specific query type
      mCaelumQueryFlags = 0xF0000000;
    }
    // Set the camera
    void setCamera (Camera *camera);
    // Place in the world
    void setLocation (Real latitude, Real longitude);
    // Get location
    Vector3 getLocation (void);
    // Set time
    void setTime (unsigned int hour, unsigned int minute, unsigned int second);
    // Get time
    Real getTime (void);
    // Get haze
    CaelumHaze *getHaze (void);
    // Get dome
    CaelumDome *getDome (void);
    // Update Caelum in time
    bool update (Real time);
  protected:
    // Scene manager
    SceneManager *mSceneMgr;
    // Current camera
    Camera *mCamera;
    // Haze
    CaelumHaze *mHaze;
    // Dome
    CaelumDome *mDome;
    // Sun
    CaelumSun *mSun;
    // Scale (in metres)
    Real mMetreUnit;
    // Geographic coordinates -- ignore altitude (z)
    Vector3 mLocation;
    // Local time
    Real mTime;
    // Update notifier flags
    unsigned int mUpdateFlags;
    // Caelum query flags
    unsigned long int mCaelumQueryFlags;
};

#endif