/*
 * Caelum dome
 */
#include "CaelumDome.h"
#include "CaelumManager.h"

//----------------------------------------------------------------------
CaelumDome::CaelumDome (CaelumManager *parent) {
  mParent = parent;
  //
  mColourMap = 0;
  mColourMapData = 0;
  mColourMapDataSteps = 0;
  mColourMapDataComponents = 0;
  mColour = ColourValue (0.3, 0.3, .5);
  mPressure = 1;

  Entity *skyDomeEntity = mParent->mSceneMgr->createEntity ("CaelumSkyDome", "CaelumSkyDome.mesh");
  skyDomeEntity->setQueryFlags (mParent->mCaelumQueryFlags);

  mMaterial = static_cast<Material *>(MaterialManager::getSingleton ().create ("CaelumSkyDome"));
  // Generate Material
  Pass *pass = mMaterial->getTechnique (0)->getPass (0);
  pass->createTextureUnitState ();  // Stars
  pass->createTextureUnitState ();  // Sky
  pass->createTextureUnitState ();  // Sun
  pass->getTextureUnitState (DOME_TUS_STARS)->setTextureName ("SkyStars.png");
  pass->getTextureUnitState (DOME_TUS_STARS)->setTextureCoordSet (0);
  pass->getTextureUnitState (DOME_TUS_COLOUR)->setTextureName ("SkyDome.png");
  pass->getTextureUnitState (DOME_TUS_COLOUR)->setTextureCoordSet (1);
  pass->getTextureUnitState (DOME_TUS_COLOUR)->setColourOperation (LBO_ALPHA_BLEND);
  pass->getTextureUnitState (DOME_TUS_SUN)->setTextureName ("SunGlow.png");
  pass->getTextureUnitState (DOME_TUS_SUN)->setTextureCoordSet (0);
  pass->getTextureUnitState (DOME_TUS_SUN)->setColourOperation (LBO_ADD);
  pass->setLightingEnabled (false);
  pass->setDepthCheckEnabled (false);
  pass->setDepthWriteEnabled (false);
  pass->setSceneBlending (SBF_ONE, SBF_ZERO);

  skyDomeEntity->setMaterialName (mMaterial->getName ());
  skyDomeEntity->setRenderQueueGroup (RENDER_QUEUE_1);

  mNode = static_cast<SceneNode *>(mParent->mSceneMgr->getRootSceneNode ()->createChildSceneNode ());
  mNode->attachObject (skyDomeEntity);

  mTouched = true;
}
//------------------------------------------------------------------------------
bool CaelumDome::setColourMap (String file) {
  mColourMap = new Image ();
  if (mColourMap) {
    mColourMap->load (file);
    // PLEASE!!!! 24bit data!!!!!!!! (or 32bit!!!)
    mColourMapData = mColourMap->getData ();
    if (mColourMap->getHasAlpha ())
      mColourMapDataComponents = 4;
    else
      mColourMapDataComponents = 3;
    mColourMapDataSteps = mColourMap->getSize () / mColourMapDataComponents;

    return true;
  }

  return false;
}
//------------------------------------------------------------------------------
void CaelumDome::setColour (ColourValue &colour) {
	mColour = colour;

  mTouched = true;
}
//------------------------------------------------------------------------------
void CaelumDome::setColour (Real r, Real g, Real b) {
	mColour.r = r;
	mColour.g = g;
	mColour.b = b;

  mTouched = true;
}
//------------------------------------------------------------------------------
ColourValue &CaelumDome::getColour (void) {
	return mColour;
}
//------------------------------------------------------------------------------
void CaelumDome::setPressure (Real pressure) {
  mPressure = pressure;

  // Avoid a 'division by zero' error
  if (mPressure <= 0) {
    mPressure = 0.000001f;
  }
}
//------------------------------------------------------------------------------
Real CaelumDome::getPressure (void) {
  return mPressure;
}
//------------------------------------------------------------------------------
void CaelumDome::_update (void) {
	mNode->setPosition (mParent->mCamera->getDerivedPosition ());
	Real dist = mParent->getHaze ()->getRange ();
  mNode->setScale (dist, dist / mPressure, dist);

  // Change sky colour...
  // current position in relative daytime
  Real timePos = mParent->mTime / 86400;
  mMaterial->getTechnique (0)->getPass (0)->getTextureUnitState (DOME_TUS_STARS)->setTextureUScroll (timePos);
  mMaterial->getTechnique (0)->getPass (0)->getTextureUnitState (DOME_TUS_COLOUR)->setTextureUScroll (timePos);
  mMaterial->getTechnique (0)->getPass (0)->getTextureUnitState (DOME_TUS_SUN)->setTextureUScroll (timePos + 0.5);

  mTouched = false;
}
//------------------------------------------------------------------------------
