/*
 * Caleum Haze
 */
#include "CaelumHaze.h"
#include "CaelumManager.h"

using namespace Ogre;

//------------------------------------------------------------------------------
bool CaelumHaze::setColourMap (String file) {
  mColourMap = new Image ();
  if (mColourMap) {
    mColourMap->load (file);
    // PLEASE!!!! 24bit data!!!!!!!! (or 32bit!!!)
    mColourMapData = mColourMap->getData ();
    if (mColourMap->getHasAlpha ())
      mColourMapDataComponents = 4;
    else
      mColourMapDataComponents = 3;
    mColourMapDataSteps = mColourMap->getSize () / mColourMapDataComponents;

    return true;
  }

  return false;
}
//------------------------------------------------------------------------------
void CaelumHaze::setColour (ColourValue &colour) {
	mColour = colour;

  mTouched = true;
}
//------------------------------------------------------------------------------
void CaelumHaze::setColour (Real r, Real g, Real b) {
	mColour.r = r;
	mColour.g = g;
	mColour.b = b;

  mTouched = true;
}
//------------------------------------------------------------------------------
ColourValue &CaelumHaze::getColour (void) {
	return mColour;
}
//------------------------------------------------------------------------------
void CaelumHaze::setTurbidity (Real turbidity) {
  mTurbidity = turbidity;

  mTouched = true;
}
//------------------------------------------------------------------------------
Real CaelumHaze::getTurbidity (void) {
  return mTurbidity;
}
//------------------------------------------------------------------------------
Real CaelumHaze::getRange (void) {
  return (1 / (mParent->mMetreUnit * mTurbidity));
}
//------------------------------------------------------------------------------
void CaelumHaze::_update (void) {
  _recalculateColour ();

  if (mTouched) {
    mParent->mSceneMgr->setAmbientLight (mColour);
    mParent->mSceneMgr->setFog (FOG_LINEAR, mColour, 0, 0, getRange ());
    mParent->mCamera->setFarClipDistance (getRange ());

    mTouched = false;
  }
}
//------------------------------------------------------------------------------
void CaelumHaze::_recalculateColour (void) {
  if (mColourMap) {
    // To avoid finger-ache ;)
    unsigned short int n = mColourMapDataComponents;
    // current position in relative daytime
    Real timePos = mParent->mTime / 86400;
    // Position in the colour map
    Real mapPos = timePos * mColourMapDataSteps;
    unsigned int pos;
    Real offset;

    ColourValue c, c0, c1;
    pos = (unsigned int )mapPos * n;
    c0.r = mColourMapData[pos] / 256.0f;
    c0.g = mColourMapData[pos+1] / 256.0f;
    c0.b = mColourMapData[pos+2] / 256.0f;
    if (n == 4)
      c0.a = mColourMapData[pos+3] / 256.0f;

    offset = ((mapPos * n) - pos) / n;

    pos = (pos + n) % (mColourMapDataSteps * n);
    c1.r = mColourMapData[pos] / 256.0f;
    c1.g = mColourMapData[pos+1] / 256.0f;
    c1.b = mColourMapData[pos+2] / 256.0f;
    if (n == 4)
      c1.a = mColourMapData[pos+3] / 256.0f;

    c.r = c0.r + (c1.r - c0.r) * offset;
    c.g = c0.g + (c1.g - c0.g) * offset;
    c.b = c0.b + (c1.b - c0.b) * offset;
    if (n == 4)
      c.a = c0.a + (c1.a - c0.a) * offset;

    mColour = ColourValue (c);

    if (n == 4)
      //mTurbidity = c.a; /////////// Change this or you'll not see anything!!!
      mTurbidity = (mMaximumTurbidity - mMinimumTurbidity) * c.a + mMinimumTurbidity;

    mTouched = true;
  }
}
//------------------------------------------------------------------------------
