/*
 * CaelumManager.cpp
 */
#include "CaelumManager.h"

using namespace Ogre;

// Set the current camera
void CaelumManager::setCamera (Camera *camera) {
  mCamera = camera;
}
// Set Location
void CaelumManager::setLocation (Real latitude, Real longitude) {
  while (longitude > 180)
    longitude -= 360;
  while (longitude < -180)
    longitude += 360;

  while (latitude > 90)
    latitude -= 180;
  while (latitude < -90)
    latitude += 180;

  mLocation.x = longitude;
  mLocation.y = latitude;
  mLocation.z = 0;
}
// Get location
Vector3 CaelumManager::getLocation (void) {
  return mLocation;
}
// Set time
void CaelumManager::setTime (unsigned int hour, unsigned int minute, unsigned int second) {
  mTime = (hour * 60 * 60); // Hours to seconds
  mTime += (minute * 60);   // Minutes to seconds
  mTime += second;          // Second

  // Normalize time
  while (mTime > 86400)
    mTime -= 86400;
}
// Get time
Real CaelumManager::getTime (void) {
  return mTime;
}
// Get the haze
CaelumHaze *CaelumManager::getHaze (void) {
  return mHaze;
}
// Get the dome
CaelumDome *CaelumManager::getDome (void) {
  return mDome;
}
// Update the atmosphere
bool CaelumManager::update (Real time) {
  mTime += time;

  // Normalize time
  while (mTime > 86400)
    mTime -= 86400;

  mHaze->_update ();
  mDome->_update ();
  mSun->_update ();

  return true;
}
