/*
 * Caelum Manager
 * Author: Kencho
 * Starting date: 04-15-2004
 * Last update: 04-15-2004
 */
#ifndef __CAELUMMANAGER_H__
#define __CAELUMMANAGER_H__

#include "Ogre.h"

#include "CaelumDome.h"
#include "CaelumHaze.h"
#include "CaelumObject.h"

namespace Ogre {
  typedef std::map<String, CaelumObject *> CaelumObjectList;

	/* Manages Caelum */
  class CaelumManager {
		public:
      // Constructor
      CaelumManager (SceneManager *sceneMgr, Camera *camera);
      // Destructor
      ~CaelumManager ();
      // SceneManager rebinding
      void setSceneManager (SceneManager *sceneMgr);
      // SceneManager query
      SceneManager *getSceneManager (void);
      // Camera rebinding
      void setCamera (Camera *camera);
      // Camera query
      Camera *getCamera (void);
      // Get Dome
      CaelumDome *getDome (void);
      // Get Haze
      CaelumHaze *getHaze (void);
      // Get Object
      CaelumObject *getObject (String name);
      // Create a new object
      CaelumObject *createObject (String name, MovableObject *content);
      // Time set
      void setTime (Real time);
      void setTime (unsigned int hour = 0, unsigned int minute = 0, unsigned int second = 0);
      // Time get
      Real getTime (void);
      // ---------------------------------------- date manipulation!!!!!!!!
      // Set location
      void setLocation (Vector3 location);
      void setLocation (Real longitude, Real latitude, Real radius = 6371000);
      // Get location
      Vector3 getLocation (void);
      // Set rotation axis
      void setAxis (Vector3 axis);
      void setAxis (Real x, Real y, Real z);
      // Get rotation axis
      Vector3 getAxis (void);
      // Update caelum system :)
      void update (Real time);
		protected:
      // Scene manager this object is associated to
      SceneManager *mSceneMgr;
      // Camera this object is associated to
      Camera *mCamera;
      // Caelum objects:
      // Dome
      CaelumDome *mDome;
      // Haze
      CaelumHaze *mHaze;
      // Objects
      CaelumObjectList mObjectList;
      // Current daytime
      Real mTime;
      // Current date
      unsigned long int mDay;
      // Lcation
      Vector3 mLocation;
      // Unit scale transformation -- Relation between Ogre units and one metre
      Real mUnitScale;
      // Rotation axis of this planet
      Vector3 mAxis;
  };
}

#endif