/*
 * Caelum Dome
 * Author: Kencho
 * Starting date: 04-15-2004
 * Last update: 04-15-2004
 */
#include "CaelumDome.h"

namespace Ogre {
  //-------------------------------------------------------------------------------------
  CaelumDome::CaelumDome (SceneManager *sceneMgr, SceneNode *parentNode) {
    // Create node
    assert (parentNode && "Must pass a parent node!!!");
    mDomeNode = static_cast<SceneNode *>(parentNode->createChildSceneNode ("CaelumDomeNode"));

    mMaterial = static_cast<Material *>(MaterialManager::getSingleton ().create ("Caelum/SkyDome"));
    assert (mMaterial && "Material wasn't created!!!");
    // Generate Material
    mMaterial->setDepthCheckEnabled (false);
    mMaterial->setDepthWriteEnabled (false);
    mMaterial->setDiffuse (0, 0, 0);
    mMaterial->setSpecular (0, 0, 0);
    mMaterial->setSelfIllumination (0, 0, 0);

    mMaterial->getTechnique (0)->getPass (0)->removeAllTextureUnitStates ();

    Entity *skyDomeEntity = sceneMgr->createEntity ("CaelumSkyDome", "CaelumSkyDome.mesh");

    skyDomeEntity->setMaterialName (mMaterial->getName ());
    skyDomeEntity->setRenderQueueGroup (RENDER_QUEUE_1);

    mDomeNode->attachObject (skyDomeEntity);

    // Init attributes
    mPressure = 1;
  }
  //-------------------------------------------------------------------------------------
  CaelumDome::~CaelumDome (void) {
    // Delete the created scene node...
    assert (mDomeNode && "Dome should have a scene node...");
    static_cast<SceneNode *>(mDomeNode->getParent ())->removeAndDestroyChild (mDomeNode->getName ());
    assert (mMaterial && "There should be a material...");
    MaterialManager::getSingleton ().unload (mMaterial);
  }
  //-------------------------------------------------------------------------------------
  SceneNode *CaelumDome::getSceneNode (void) {
    return mDomeNode;
  }
  //-------------------------------------------------------------------------------------
  void CaelumDome::setPressure (Real pressure) {
    mPressure = pressure;
  }
  //-------------------------------------------------------------------------------------
  Real CaelumDome::getPressure (void) {
    return mPressure;
  }
  //-------------------------------------------------------------------------------------
  void CaelumDome::update (Real time, Vector3 camPos, Real range) {
    ColourValue c;

    // Update colour
    updateColour (time);

    // Get processed colour
    c = getFinalColour ();

    // Place node
    mDomeNode->setPosition (camPos);
    mDomeNode->setScale (range, range / mPressure, range);

    // Change node material
    if (mMaterial) {
      mMaterial->setAmbient (c);
    }
  }
  //-------------------------------------------------------------------------------------
}