/*
 * Caelum Dynacolour
 * Author: Kencho
 * Starting date: 04-15-2004
 * Last update: 04-15-2004
 */
#include "CaelumDynaColour.h"

namespace Ogre {
  //-------------------------------------------------------------------------------------
  CaelumDynaColour::CaelumDynaColour (void) {
    // Init attributes
    mColour = ColourValue::White;
    mColourMap = 0;
    mColourMapData = 0;
    mColourMapSteps = 0;
    mColourMapComponents = 0;
    mColourModifier = ColourValue::White;
    mSaturationModifier = 1;
  }
  //-------------------------------------------------------------------------------------
  CaelumDynaColour::~CaelumDynaColour (void) {
  }
  //-------------------------------------------------------------------------------------
  void CaelumDynaColour::setColour (ColourValue colour) {
    mColour = colour;
  }
  //-------------------------------------------------------------------------------------
  ColourValue CaelumDynaColour::getColour (void) {
    return mColour;
  }
  //-------------------------------------------------------------------------------------
  void CaelumDynaColour::setColourMap (Image *image) {
    mColourMap = image;
    if (mColourMap) {
      mColourMapData = mColourMap->getData ();
      if (mColourMap->getHasAlpha ()) {
        mColourMapComponents = 4;
      }
      else {
        mColourMapComponents = 3;
      }
      mColourMapSteps = mColourMap->getSize () / mColourMapComponents;
    }
    else {
      mColourMapData = 0;
      mColourMapComponents = 0;
      mColourMapSteps = 0;
    }
  }
  //-------------------------------------------------------------------------------------
  Image *CaelumDynaColour::getColourMap (void) {
    return mColourMap;
  }
  //-------------------------------------------------------------------------------------
  void CaelumDynaColour::updateColour (Real time) {
    if (mColourMap) {
      // To avoid finger-ache ;)
      unsigned short int n = mColourMapComponents;
      // Position in the colour map
      Real mapPos = time * mColourMapSteps;

      ColourValue c, c0, c1;
      unsigned int pos = (unsigned int )mapPos * n;
      c0.r = mColourMapData[pos] / 256.0f;
      c0.g = mColourMapData[pos+1] / 256.0f;
      c0.b = mColourMapData[pos+2] / 256.0f;
      if (n == 4)
        c0.a = mColourMapData[pos+3] / 256.0f;

      Real offset = ((mapPos * n) - pos) / n;

      pos = (pos + n) % (mColourMapSteps * n);
      c1.r = mColourMapData[pos] / 256.0f;
      c1.g = mColourMapData[pos+1] / 256.0f;
      c1.b = mColourMapData[pos+2] / 256.0f;
      if (n == 4)
        c1.a = mColourMapData[pos+3] / 256.0f;

      c.r = c0.r + (c1.r - c0.r) * offset;
      c.g = c0.g + (c1.g - c0.g) * offset;
      c.b = c0.b + (c1.b - c0.b) * offset;
      if (n == 4)
        c.a = c0.a + (c1.a - c0.a) * offset;

      mColour = c;
    }
  }
  //-------------------------------------------------------------------------------------
  void CaelumDynaColour::setColourModifier (ColourValue colour) {
    mColourModifier = colour;
  }
  //-------------------------------------------------------------------------------------
  ColourValue CaelumDynaColour::getColourModifier (void) {
    return mColourModifier;
  }
  //-------------------------------------------------------------------------------------
  void CaelumDynaColour::setSaturationModifier (Real saturation) {
    mSaturationModifier = saturation;
  }
  //-------------------------------------------------------------------------------------
  Real CaelumDynaColour::getSaturationModifier (void) {
    return mSaturationModifier;
  }
  //-------------------------------------------------------------------------------------
  ColourValue CaelumDynaColour::getFinalColour (void) {
    // Needed vars
    ColourValue colour;
    Real lightness;

    // Base colour
    colour = mColour;

    // Multiplied colour
    colour.r *= mColourModifier.r;
    colour.g *= mColourModifier.g;
    colour.b *= mColourModifier.b;

    // Current colour lightness
    lightness = (colour.r + colour.g + colour.b) / 3.0;

    // (De)Saturated colour
    colour.r = lightness + (colour.r - lightness) * mSaturationModifier;
    colour.g = lightness + (colour.g - lightness) * mSaturationModifier;
    colour.b = lightness + (colour.b - lightness) * mSaturationModifier;

    // Return preocessed colour
    return colour;
  }
  //-------------------------------------------------------------------------------------
}