/*
 * Caelum Object
 * Author: Kencho
 * Starting date: 04-16-2004
 * Last update: 04-16-2004
 */
#include "CaelumObject.h"

namespace Ogre {
  //-------------------------------------------------------------------------------------
  CaelumObject::CaelumObject (String name, SceneNode *parent) {
    mSceneNode = parent->createChildSceneNode (name);
    mLight = parent->getCreator ()->createLight (name);
    mLight->setType (Light::LT_DIRECTIONAL);
    mSceneNode->attachLight (mLight);
    mPeriod = 86400;
    mTimeOffset = 0;

    CaelumDynaColour::CaelumDynaColour ();
  }
  //-------------------------------------------------------------------------------------
  CaelumObject::~CaelumObject (void) {
  }
  //-------------------------------------------------------------------------------------
  void CaelumObject::setSceneNode (SceneNode *node) {
    mSceneNode = node;
  }
  //-------------------------------------------------------------------------------------
  SceneNode *CaelumObject::getSceneNode (void) {
    return mSceneNode;
  }
  //-------------------------------------------------------------------------------------
  void CaelumObject::setTimeOffset (Real offset) {
    mTimeOffset = offset;
  }
  //-------------------------------------------------------------------------------------
  Real CaelumObject::getTimeOffset (void) {
    return mTimeOffset;
  }
  //-------------------------------------------------------------------------------------
  void CaelumObject::update (Real time, Vector3 camPos, Vector3 scale) {
    assert (mSceneNode && "Can't update if there is no scene node!!!");

//    Vector3 size = mSceneNode->getParent ()->getScale ();
//    Vector3 size = Vector3 (200, 200, 200);
    Vector3 size = scale;
    mSceneNode->setPosition (Vector3 (-Math::Cos ((time + mTimeOffset) * 2 * Math::PI), Math::Sin ((time + mTimeOffset) * 2 * Math::PI), 0) * size - mSceneNode->getParent ()->_getDerivedPosition () + camPos);

    updateColour (time);

    mLight->setDiffuseColour (getFinalColour ());
    mLight->setDirection (camPos - mSceneNode->getPosition ());

    Material *mat = mSceneNode->getMaterial ();
    mat->setAmbient (getFinalColour ());
    mat->setDiffuse (ColourValue::Black);
    mat->setSpecular (ColourValue::Black);
    mat->setSelfIllumination (ColourValue::Black);
  }
  //-------------------------------------------------------------------------------------
}